<?php

namespace Drupal\server_sent_events;

use Drupal\Core\Entity\EntityAccessControlHandler;
use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Session\AccountInterface;
use Drupal\Core\Access\AccessResult;

/**
 * Access controller for the Channel subscription entity.
 *
 * @see \Drupal\server_sent_events\Entity\ChannelSubscription.
 */
class ChannelSubscriptionAccessControlHandler extends EntityAccessControlHandler {

  /**
   * {@inheritdoc}
   */
  protected function checkAccess(EntityInterface $entity, $operation, AccountInterface $account) {
    /** @var \Drupal\server_sent_events\Entity\ChannelSubscriptionInterface $entity */

    switch ($operation) {

      case 'view':

        if (!$entity->isPublished()) {
          return AccessResult::allowedIfHasPermission($account, 'view unpublished channel subscription entities');
        }


        return AccessResult::allowedIfHasPermission($account, 'view published channel subscription entities');

      case 'update':

        return AccessResult::allowedIfHasPermission($account, 'edit channel subscription entities');

      case 'delete':

        return AccessResult::allowedIfHasPermission($account, 'delete channel subscription entities');
    }

    // Unknown operation, no opinion.
    return AccessResult::neutral();
  }

  /**
   * {@inheritdoc}
   */
  protected function checkCreateAccess(AccountInterface $account, array $context, $entity_bundle = NULL) {
    return AccessResult::allowedIfHasPermission($account, 'add channel subscription entities');
  }


}
