<?php

namespace Drupal\server_sent_events\Entity;

use Drupal\Core\Config\Entity\ConfigEntityBase;

/**
 * Defines the Events channel entity.
 *
 * @ConfigEntityType(
 *   id = "events_channel",
 *   label = @Translation("Events channel"),
 *   handlers = {
 *     "view_builder" = "Drupal\Core\Entity\EntityViewBuilder",
 *     "list_builder" = "Drupal\server_sent_events\EventsChannelListBuilder",
 *     "form" = {
 *       "add" = "Drupal\server_sent_events\Form\EventsChannelForm",
 *       "edit" = "Drupal\server_sent_events\Form\EventsChannelForm",
 *       "delete" = "Drupal\server_sent_events\Form\EventsChannelDeleteForm"
 *     },
 *     "route_provider" = {
 *       "html" = "Drupal\server_sent_events\EventsChannelHtmlRouteProvider",
 *     },
 *   },
 *   config_prefix = "events_channel",
 *   admin_permission = "administer site configuration",
 *   entity_keys = {
 *     "id" = "id",
 *     "label" = "label",
 *     "path" = "path",
 *     "subscribe" = "subscribe",
 *     "push" = "push",
 *     "status" = "status",
 *     "locked" = "locked",
 *     "uuid" = "uuid"
 *   },
 *   config_export = {
 *     "id",
 *     "label",
 *     "path",
 *     "subscribe",
 *     "push",
 *     "status",
 *     "locked",
 *     "uuid"
 *   },
 *   links = {
 *     "canonical" = "/admin/structure/events_channel/{events_channel}",
 *     "add-form" = "/admin/structure/events_channel/add",
 *     "edit-form" = "/admin/structure/events_channel/{events_channel}/edit",
 *     "delete-form" = "/admin/structure/events_channel/{events_channel}/delete",
 *     "collection" = "/admin/structure/events_channel"
 *   }
 * )
 */
class EventsChannel extends ConfigEntityBase implements EventsChannelInterface {

  /**
   * The Events channel ID.
   *
   * @var string
   */
  protected $id;

  /**
   * The Events channel label.
   *
   * @var string
   */
  protected $label;

  public function subscribe(){
    $subscription = ChannelSubscription::create([
      'channel' => $this->id(),
    ]);
    $subscription->save();
    $this->pushEvent("Subscription ID : ".$subscription->id(),'subscribe');
    return $subscription;
  }

  public function getSubscribers(){
    $subscriptions = \Drupal::entityQuery('channel_subscription')->condition('channel',$this->id())->execute();
    return $subscriptions;
  }

  public function getUserSubscriptions($user){
    $subscriptions = \Drupal::entityQuery('channel_subscription')
      ->condition('channel',$this->id())
      ->condition('user_id',$user->id())
      ->execute();
    return $subscriptions;
  }

  public function getPendingEvents($subscription){
    $eventIds = \Drupal::entityQuery('server_event_entity')
      ->condition('recipients',$subscription->id())
      ->condition('channel',$this->id())
      ->execute();
    $events = [];
    foreach ($eventIds as $eventId){
      $event = SSEventEntity::load($eventId);
      $event->deliverEvent($subscription);
      $event->save();
      $events[] = $event;
    }
    return $events;
  }

  public function pushEvent($event,$type = 'message'){
    $recipientIds = $this->getSubscribers();
    $recipientIds = array_values($recipientIds);
    $recipients = [];
    foreach($recipientIds as $recpient){
      $recipients[] = ['target_id' => $recpient];
    }
    $event = SSEventEntity::create([
      'channel' => $this->id(),
      'recipients' => $recipients,
      'type' => $type,
      'body' => $event,
    ]);
    $event->save();
    return $event;
  }

  public function checkStatus($subscription){
    echo ' ';
    ob_flush();
    flush();
    if(connection_aborted()){
      $this->unsubscribe($subscription);
      exit();
    }
  }

  public function unsubscribe($subscription){
    $subscription->delete();
    $this->pushEvent('Subscription terminated :' . $subscription->id(),'unsubscribe');
  }
}
