<?php

namespace Drupal\server_sent_events\Form;

use Drupal\Core\Entity\EntityForm;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Url;
use Drupal\facets\Exception\Exception;
use Symfony\Component\Routing\Exception\ResourceNotFoundException;

/**
 * Class EventsChannelForm.
 */
class EventsChannelForm extends EntityForm {

  /**
   * {@inheritdoc}
   */
  public function form(array $form, FormStateInterface $form_state) {
    $form = parent::form($form, $form_state);

    $events_channel = $this->entity;
    $form['label'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Label'),
      '#maxlength' => 255,
      '#default_value' => $events_channel->label(),
      '#description' => $this->t("Label for the Events channel."),
      '#required' => TRUE,
    ];

    $form['id'] = [
      '#type' => 'machine_name',
      '#default_value' => $events_channel->id(),
      '#machine_name' => [
        'exists' => '\Drupal\server_sent_events\Entity\EventsChannel::load',
      ],
      '#disabled' => !$events_channel->isNew(),
    ];

    $form['path'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Path'),
      '#maxlength' => 255,
      '#default_value' => $events_channel->get('path'),
      '#description' => $this->t("Path for the Events channel."),
      '#required' => TRUE,
    ];
    $permissions = array_keys(\Drupal::service('user.permissions')->getPermissions());
    $permissions = array_combine($permissions,$permissions);
    $form['subscribe'] = [
      '#type' => "select",
      '#title' => $this->t("Subscribe permission"),
      '#default_value' => $events_channel->get('subscribe'),
      '#description' => $this->t('Permission to subscribe to the channel'),
      '#required' => TRUE,
      '#options' => $permissions,
    ];

    $form['push'] = [
      '#type' => "select",
      '#title' => $this->t("Push permission"),
      '#default_value' => $events_channel->get('push'),
      '#description' => $this->t('Permission to push events to the channel'),
      '#required' => TRUE,
      '#options' => $permissions,
    ];
    $form['status'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Active'),
      '#default_value' => $events_channel->get('status'),
    ];
    if(!$events_channel->isNew()){
      $form['test_push'] = [
        '#type' => 'textarea',
        '#title' => $this->t("Push to Channel"),
        '#rows' => 2,
        '#suffix' => "<a class='test-push button btn'>Test push</a>",
      ];
      $form['subscribers'] = [
        '#type' => 'details',
        '#title' => $this->t('Subscribers'),
        'subscribers_container' => [
          '#type' => 'container',
          '#attributes' => [
            'class' => ['subscribers-container'],
          ],
        ],
      ];
      $form['live_show'] = [
        '#type' => 'details',
        "#title" => $this->t("Live feed"),
        '#open' => TRUE,
        'live' => [
          '#type' => 'container',
          '#attributes' => [
            'class' => ['channel-live-feed']
          ]
        ],
      ];
      $form['#attached']['drupalSettings']['server_sent_events']['channel'] = [
        "channel_path" => $events_channel->get('path'),
        "push_path" => Url::fromRoute('server_sent_events.channel.push.'.$events_channel->id())->toString(),
        "subscribers_path" => Url::fromRoute('server_sent_events.channel.subscribers.'.$events_channel->id())->toString(),
      ];
      //$form['#attached']['drupalSettings']['server_sent_events']['channel_path'] = $events_channel->get('path');
      $form['#attached']['library'][] = 'server_sent_events/server_sent_events';
      $form['#attached']['library'][] = 'server_sent_events/server_sent_events_admin';

    }
    /* You will need additional form elements for your custom properties. */

    return $form;
  }

  public function validateForm(array &$form, FormStateInterface $form_state) {
    parent::validateForm($form, $form_state);
    $path = '/' . trim($form_state->getUserInput()['path'],'/');
    $router = \Drupal::service('router.no_access_checks');
    try{
      $result = $router->match($path);
    }
    catch (ResourceNotFoundException $exception){
      $result = [];
    }
    if(count($result) != 0){
      $parts = explode('.' ,$result['_route']);
      if($parts[0] != 'server_sent_events' || $parts[1] != 'channel'){
        $form_state->setErrorByName('path',$this->t('This path is already used by route @routename',['@routename'=>$result['_route']]));
      }
      else{
        $form_state->setValue('path',$path);
      }
    }
    else{
      $form_state->setValue('path',$path);
    }
  }

  /**
   * {@inheritdoc}
   */
  public function save(array $form, FormStateInterface $form_state) {
    $events_channel = $this->entity;
    $events_channel->set('locked',0);
    $status = $events_channel->save();

    switch ($status) {
      case SAVED_NEW:
        $this->messenger()->addMessage($this->t('Created the %label Events channel.', [
          '%label' => $events_channel->label(),
        ]));
        break;

      default:
        $this->messenger()->addMessage($this->t('Saved the %label Events channel.', [
          '%label' => $events_channel->label(),
        ]));
    }
    $form_state->setRedirectUrl($events_channel->toUrl('collection'));
    \Drupal::service('router.builder')->setRebuildNeeded();
  }

}
