<?php

namespace Drupal\server_sent_events\Form;

use Drupal\Core\Database\Database;
use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;

/**
 * Configure Server Sent Events settings for this site.
 */
class SettingsForm extends ConfigFormBase {

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'server_sent_events_settings';
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return ['server_sent_events.settings'];
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $t = \Drupal::service('server_sent_events.service')->getEventDataSources();
    $options = [];
    foreach($t as $source){
      if($source['type'] == 'filesystem'){
        $options[$source['value']] = $source['name'];
      }
      elseif($source['type'] == 'database'){
        $options[$source['value']] = $source['name'];
      }
    }
    $default_value = $this->config('server_sent_events.settings')->get('engine');
    if($default_value == '') {
      $default_value = 'db:default';
    }
    $form['engine'] = [
      '#type' => 'radios',
      '#title' => 'Events data source',
      '#default_value' => $default_value,
      '#options' => $options,
    ];
    //dd(\Drupal::service('server_sent_events.service')->getEventSource());
    return parent::buildForm($form, $form_state);
  }
  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {

    parent::validateForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $this->config('server_sent_events.settings')
      ->set('engine', $form_state->getValue('engine'))
      ->save();
    parent::submitForm($form, $form_state);
  }

}
