<?php
namespace Drupal\server_sent_events\Routing;
use Drupal\Core\Routing\RouteSubscriberBase;
use Symfony\Component\Routing\Route;
use Symfony\Component\Routing\RouteCollection;


/**
 * Defines dynamic routes.
 */
class ServerSentEventsRouting extends RouteSubscriberBase{

  public function __construct(){
    $this->storage = \Drupal::entityTypeManager()->getStorage('events_channel');
  }

  /**
   * {@inheritdoc}
   */
  public function alterRoutes(RouteCollection $collection) {
    $routes = [];
    $channelIDs = \Drupal::entityQuery('events_channel')->condition('status',1)->execute();
    foreach($channelIDs as $channelID){
      $channel = $this->storage->load($channelID);
      $route = new Route(
        $channel->get('path'),
        [
          '_controller' => '\Drupal\server_sent_events\Controller\ServerSentEventsController::channel',
        ],
        [
          '_permission'  => $channel->get('subscribe'),
        ]
      );
      $collection->add('server_sent_events.channel.'.$channelID,$route);
      $pushPath = "/server-sent-events/push/".$channelID;
      $pushRoute = new Route(
        $pushPath,
        [
          '_controller' => '\Drupal\server_sent_events\Controller\ServerSentEventsController::pushMessage',
        ],
        [
          '_permission'  => $channel->get('push'),
        ]
      );
      $collection->add('server_sent_events.channel.push.'.$channelID,$pushRoute);

      $subscribersPath = "/server-sent-events/subscribers/".$channelID;
      $subscribersRoute = new Route(
        $subscribersPath,
        [
          '_controller' => '\Drupal\server_sent_events\Controller\ServerSentEventsController::getSubscribers',
        ],
        [
          '_permission'  => $channel->get('subscribe'),
        ]
      );
      $collection->add('server_sent_events.channel.subscribers.'.$channelID,$subscribersRoute);
    }
  }

}
