<?php

namespace Drupal\server_sent_events;

use Drupal\Core\Entity\EntityAccessControlHandler;
use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Session\AccountInterface;
use Drupal\Core\Access\AccessResult;

/**
 * Access controller for the Server Sent Event Entity entity.
 *
 * @see \Drupal\server_sent_events\Entity\SSEventEntity.
 */
class SSEventEntityAccessControlHandler extends EntityAccessControlHandler {

  /**
   * {@inheritdoc}
   */
  protected function checkAccess(EntityInterface $entity, $operation, AccountInterface $account) {
    /** @var \Drupal\server_sent_events\Entity\SSEventEntityInterface $entity */

    switch ($operation) {

      case 'view':

        if (!$entity->isPublished()) {
          return AccessResult::allowedIfHasPermission($account, 'view unpublished server sent event entity entities');
        }


        return AccessResult::allowedIfHasPermission($account, 'view published server sent event entity entities');

      case 'update':

        return AccessResult::allowedIfHasPermission($account, 'edit server sent event entity entities');

      case 'delete':

        return AccessResult::allowedIfHasPermission($account, 'delete server sent event entity entities');
    }

    // Unknown operation, no opinion.
    return AccessResult::neutral();
  }

  /**
   * {@inheritdoc}
   */
  protected function checkCreateAccess(AccountInterface $account, array $context, $entity_bundle = NULL) {
    return AccessResult::allowedIfHasPermission($account, 'add server sent event entity entities');
  }


}
